package com.bizmatics.mhfire.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.common.core.util.DateUtils;
import com.bizmatics.mhfire.persistence.mapper.FireStatisticsMapper;

import com.bizmatics.mhfire.persistence.mapper.po.FireStatisticsPO;
import com.bizmatics.mhfire.persistence.mapper.po.UnitPO;
import com.bizmatics.mhfire.service.FireStatisticsService;
import com.bizmatics.mhfire.service.UnitService;

import com.bizmatics.mhfire.service.vo.FireLevelRatioVO;
import com.bizmatics.mhfire.service.vo.FireUnitPoliceVO;
import org.checkerframework.checker.nullness.Opt;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author yq
 * @date 2021/5/28 9:35
 */
@Service
public class FireStatisticsServiceImpl implements FireStatisticsService {

    @Autowired
    private FireStatisticsMapper fireStatisticsMapper;
    @Autowired
    private UnitService unitService;
    @Override
    public List<Integer> getFireCountByData(Integer year, String address,String fireType) {
        List<Integer> list = new ArrayList<>();
        Date date = new Date();
        for (int i = 0; i < 12; i++) {
            date.setDate(1);
            Date setMonths = DateUtils.setMonths(date, i);
            Date firstDayOfMonth = DateUtils.getFirstDayOfMonth(setMonths);
            Date lastDayOfMonth = DateUtils.getLastDayOfMonth(setMonths);
            list.add(fireStatisticsMapper.selectCount(firstDayOfMonth, lastDayOfMonth, address,fireType,null));
        }
        return list;
    }


    @Override
    public CommonPage<FireStatisticsPO> page(Integer current, Integer size, Date startTime, Date endTime, String address) {
        Page<Map<String, String>> page = new Page<>(current, size);
        page = fireStatisticsMapper.page(page, startTime, endTime, address);
        List<FireStatisticsPO> list = new ArrayList<>();
        page.getRecords().forEach(stringObjectMap -> list.add(enhanceFireStatisticsPo(stringObjectMap)));
        return new CommonPage<>(list, page.getTotal(), page.getSize(), page.getCurrent());
    }

    @Override
    public FireStatisticsPO getOne(String id) {
        return enhanceFireStatisticsPo(fireStatisticsMapper.getOne(id));
    }

    @Override
    public FireUnitPoliceVO getFireUnitPolice(String id) {
        //火灾详细信息
        FireStatisticsPO fireStatisticsPo = getOne(id);
        //单位信息
        UnitPO unitPo = new UnitPO();
        unitPo.setAddress(fireStatisticsPo.getAddress());
        unitPo = unitService.getOne(unitPo);

        //TODO 出警记录没有
        FireUnitPoliceVO fireUnitPoliceVO = new FireUnitPoliceVO();
        fireUnitPoliceVO.setFireStatisticsPo(fireStatisticsPo);
        fireUnitPoliceVO.setUnitPo(unitPo);
        return fireUnitPoliceVO;
    }

    @Override
    public List<FireLevelRatioVO> getGroupByLevel(Date startTime, Date endTime, String address) {
        List<FireLevelRatioVO> list = new ArrayList<>();
        int fireCount = 0;
        List<String> fireCauses = fireStatisticsMapper.groupByCause();
        for (String cause:fireCauses) {
            FireLevelRatioVO fireLevelRatioVo = new FireLevelRatioVO();
            fireLevelRatioVo.setFireType(cause);
            Integer count = fireStatisticsMapper.selectCount(startTime, endTime, address, cause,null);
            fireLevelRatioVo.setRadio(count.doubleValue());
            list.add(fireLevelRatioVo);
            fireCount+=count;
        }
        //排序
        List<FireLevelRatioVO> orderByList = list.stream().sorted(Comparator.comparing(FireLevelRatioVO::getRadio).reversed()).limit(10).collect(Collectors.toList());

        //差集求和
        double sum = list.stream().filter(item -> !orderByList.contains(item)).mapToDouble(FireLevelRatioVO::getRadio).sum();
        FireLevelRatioVO fireLevelRatioVO = orderByList.stream().filter(item -> "其他".equals(item.getFireType())).findAny().orElse(null);
        fireLevelRatioVO = Optional.ofNullable(fireLevelRatioVO).map(fireLevelRatio -> {
                    fireLevelRatio.setRadio(fireLevelRatio.getRadio() + sum);
                    return fireLevelRatio;
                }
        ).orElse(new FireLevelRatioVO());
        if (null == fireLevelRatioVO.getFireType()){
            fireLevelRatioVO.setFireType("其他");
            fireLevelRatioVO.setRadio(sum);
            orderByList.add(fireLevelRatioVO);
        }
        int finalFireCount = fireCount;
        if (0 != finalFireCount){
            orderByList.forEach(fireLevelRatio -> fireLevelRatio.setRadio(fireLevelRatio.getRadio()/finalFireCount));
        }
        return orderByList;
    }


    @Override
    public List<String> getFireType() {
        return fireStatisticsMapper.groupByCause();
    }

    @Override
    public List<String> getAddress() {
        return fireStatisticsMapper.getAddress();
    }

    public FireStatisticsPO enhanceFireStatisticsPo(Map<String,String> map){
        FireStatisticsPO fireStatisticsPo = new FireStatisticsPO();
        fireStatisticsPo.setId(map.get("火灾序号"));
        fireStatisticsPo.setAddress(map.get("起火地点"));
        fireStatisticsPo.setBurnedArea(Optional.ofNullable(map.get("过火面积")).orElse(""));
        fireStatisticsPo.setPropertyLoss(Optional.ofNullable(map.get("直接财产损失")).orElse(""));
        fireStatisticsPo.setDeathToll(map.get("死亡人数"));
        fireStatisticsPo.setNonFatal(Optional.ofNullable(map.get("受灾人数")).orElse(""));
        fireStatisticsPo.setDisasterHome(Optional.ofNullable(map.get("受灾户数")).orElse(""));
        fireStatisticsPo.setFireCause(Optional.ofNullable(map.get("火灾原因分类（一级）")).orElse(""));
        return fireStatisticsPo;
    }
}
